--[[
	Copyright (C) Udorn (Blackhand)
--]]

local viewType = "QuestEntry"
local version = 1
local YakaGui = LibStub("YakaGui", true)
local info = YakaGui:GetViewInfo(viewType) 
if (info and info.version >= version) then 
	return 
end

qmstr = qmstr or {}
qmstr.QuestEntry = {}
qmstr.QuestEntry.prototype = {}
qmstr.QuestEntry.metatable = {__index = qmstr.QuestEntry.prototype}
setmetatable(qmstr.QuestEntry.prototype, {__index = yaka.Container.prototype})

qmstr.QuestEntry.info = {viewType = viewType, version = version}
qmstr.QuestEntry.GetViewInfo = function()
	return qmstr.QuestEntry.info
end

qmstr.QuestEntry.childViews = {}
qmstr.QuestEntry.questIconViews = {}
qmstr.QuestEntry.iconViews = {}
qmstr.QuestEntry.spacerViews = {}
qmstr.QuestEntry.checkboxViews = {}

local ICON_SIZE = 24
local ICON_PAD_TOP = 2
local WHITE_FONT_COLOR_CODE = "|cffffffff"

local log = vendor.Debug:new("QuestEntry")

-- TODO highlight wtitle when entering mouse

-- apparently local for QuestPOI.lua
local function _QuestPOI_FindButtonByQuestId(parentName, questId)
	local poiButton
	local numButtons = 20
	local poiType = 1
	
	for j = 1, numButtons do
		poiButton = _G["poi"..parentName..poiType.."_"..j]
		if (poiButton and poiButton.questId == questId) then
			return poiButton
		end
	end
end

local function _ToggleQuest(questIndex)
	if (QuestLogFrame:IsShown()) then
		if (questIndex == GetQuestLogSelection()) then
			HideUIPanel(QuestLogFrame)
			return
		end
	end
	ShowUIPanel(QuestLogFrame)
	SelectQuestLogEntry(questIndex)
	QuestLog_SetSelection(questIndex)
	QuestLog_Update()
end

local function _ToggleTrack(questId)
	if (questId == GetSuperTrackedQuestID()) then
		SetSuperTrackedQuestID(0)
		--WorldMapFrame_SelectQuestById(0)
		WORLDMAP_SETTINGS.selectedQuestId = 0
		--WorldMapTrackQuest_Toggle(false)
	else
		SetSuperTrackedQuestID(questId)
		--WorldMapFrame_SelectQuestById(questId)
		WORLDMAP_SETTINGS.selectedQuestId = questId
		--WorldMapTrackQuest_Toggle(true)
	end
	--WatchFrame_Update()
end

local function _OnQuestClick(info)
	local self = info.arg
	local questId = self.cfg.questId
	local questIndex = self.cfg.questIndex
	--SelectQuestLogEntry(info.arg.questId)
	if ("RightButton" == info.button) then
		_ToggleTrack(questId)
	else
		if (IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow()) then
			local questLink = GetQuestLink(questIndex)
			if (questLink) then
				ChatEdit_InsertLink(questLink)
			end
		else
			--ExpandQuestHeader(GetQuestSortIndex(questIndex)
			QuestLog_OpenToQuest(questIndex)
		end
	end
end

local function _SetActive(info)
	qmstr.QuestMaster:SetIgnoreQuest(info.view:GetId(), not info.value)
end

local function _IsActive(info)
	return not qmstr.QuestMaster:IsQuestIgnored(info.view:GetId())
end


local function _WhipeChildViews(container)
	for k,v in pairs(container:GetChildViews()) do
		if ("Text" == v:GetViewType()) then
			table.insert(qmstr.QuestEntry.childViews, v)
		elseif ("Icon" == v:GetViewType()) then
			table.insert(qmstr.QuestEntry.iconViews, v)
		elseif ("QuestIcon" == v:GetViewType()) then
			table.insert(qmstr.QuestEntry.questIconViews, v)
		elseif ("Spacer" == v:GetViewType()) then
			table.insert(qmstr.QuestEntry.spacerViews, v)
		elseif ("Checkbox" == v:GetViewType()) then
			table.insert(qmstr.QuestEntry.checkboxViews, v)
		end
	end
	container:RemoveChildViews()
end

local function _OnUpdate(info)
	local self = info.arg
	if (info.view.isHeader) then
		info.view:SetBackgroundVisibility(WORLDMAP_SETTINGS.selectedQuestId == self.cfg.questId)
	else
		info.view:SetBackgroundVisibility(false)
	end
end

local function _GetText(self, text, tooltip, paddingLeft, paddingTop, isHeader, width)
	local order = self.order
	self.order = self.order + 1
	if (#qmstr.QuestEntry.childViews > 0) then
		local view = table.remove(qmstr.QuestEntry.childViews)
		view:SetText(text)
		view:SetArg(self)
		view:SetOrder(order)
		view:SetPadding(paddingLeft, 0, paddingTop, 0)
		view:SetTooltip(tooltip)
		view:SetWidth(width or "fill")
		view.isHeader = isHeader
		return view
	end
	local view = YakaGui:CreateView({
		viewType = "Text",
		text = text,
		order = order,
		height = 12,
		width = width or "fill",
		paddingTop = paddingTop,
		paddingLeft = paddingLeft,
		onClick = _OnQuestClick,
		tooltip = tooltip,
		font = "QuestMasterNormal",
		background = {
        	file = "Interface\\Addons\\QuestMaster\\resources\\Row-Highligt",
        	texCoords = {0, 1.0, 0, 0.5},
        	vertexColor = {ORANGE_FONT_COLOR.r, ORANGE_FONT_COLOR.g, ORANGE_FONT_COLOR.b, 1}
		},
		onUpdate = _OnUpdate,
		arg = self
	})
	view.isHeader = isHeader
	return view
end

local function _GetQuestIcon(self, questIndex)
	local order = self.order
	self.order = self.order + 1
	if (#qmstr.QuestEntry.questIconViews > 0) then
		local view = table.remove(qmstr.QuestEntry.questIconViews)
		view:SetQuestIndex(questIndex)
		view:SetOrder(order)
		return view
	end
	return YakaGui:CreateView({
		viewType = "QuestIcon",
		questIndex = questIndex,
		order = order,
		height = ICON_SIZE,
		width = ICON_SIZE,
		paddingLeft = 10,
		paddingTop = ICON_PAD_TOP
	})
end

local function _GetIcon(self, texture, paddingTop)
	local order = self.order
	self.order = self.order + 1
	if (#qmstr.QuestEntry.iconViews > 0) then
		local view = table.remove(qmstr.QuestEntry.iconViews)
		view:SetOrder(order)
		view:SetNormalTexture(texture)
		view:SetPadding(20, 0, paddingTop, 0)
		return view
	end
	return YakaGui:CreateView({
		viewType = "Icon",
		order = order,
		normalTexture = {
			file = texture
		},
		height = ICON_SIZE - 10,
		width = ICON_SIZE - 10,
		paddingLeft = 10 + 10,
		paddingTop = paddingTop
	})
end

local function _GetSpacer(self)
	local order = self.order
	self.order = self.order + 1
	if (#qmstr.QuestEntry.spacerViews > 0) then
		local view = table.remove(qmstr.QuestEntry.spacerViews)
		view:SetOrder(order)
		return view
	end
	return YakaGui:CreateView({
		viewType = "Spacer",
		order = order,
		height = 1,
		width = ICON_SIZE,
		paddingLeft = 10,
		paddingTop = ICON_PAD_TOP
	})
end

local function _GetCheckbox(self, questId)
	local order = self.order
	self.order = self.order + 1
	if (#qmstr.QuestEntry.checkboxViews > 0) then
		local view = table.remove(qmstr.QuestEntry.checkboxViews)
		view:SetOrder(order)
		view:SetId(questId)
		view:SetChecked(not qmstr.QuestMaster:IsQuestIgnored(questId))
		return view
	end
	return YakaGui:CreateView({
		viewType = "Checkbox",
		order = order,
		height = 16,
		width = 16,
		id = questId,
		getValue = _IsActive,
		setValue = _SetActive,
		arg = self,
	})
end
local function _MergeColor(src, dest, percent)
	local diff = src - dest
	local rtn = src - (diff * percent)
	return math.min(1, math.max(0, rtn))
end

local function _ColorizeObjective(self, desc)
	local min = RED_FONT_COLOR
	local max = GREEN_FONT_COLOR
	local _, _, _, current, total = string.find(desc, "(.*):%s+([-%d]+)/([-%d]+)")
	if (current and total) then
		current = tonumber(current)
		total = tonumber(total)
		local percent = 0
		if (current > 0) then
			percent = current / total
		end
		local r = _MergeColor(min.r, max.r, percent) 
		local g = _MergeColor(min.g, max.g, percent)
		local b = _MergeColor(min.b, max.b, percent)
		return YakaGui:GetFontColorCode(r, g, b)
	else
		return YakaGui:GetFontColorCode(unpack(self.profile.objectiveColor))
	end
	
end

local function _CreateTooltip(title, objectives)
	return title..":\n\n"..WHITE_FONT_COLOR_CODE..objectives..FONT_COLOR_CODE_CLOSE
end

local function _GetAllObjectivesDone(self)
	local n = GetNumQuestLeaderBoards(self.cfg.questIndex)
	local allDone = true
	for i = 1, n do
		local _, _, done = GetQuestLogLeaderBoard(i, self.cfg.questIndex)
		if (not done) then
			allDone = false
			break
		end
	end
	return allDone
end

local function _WatchFrame_ReverseQuestObjective(text)
	local _, _, arg1, arg2 = string.find(text, "(.*):%s(.*)");
	if (arg1 and arg2) then
  		return arg2.." "..arg1
   	end
   	return text
end

local function _Update(self)
	-- reinit entry
	_WhipeChildViews(self.title)
	_WhipeChildViews(self.objectives)
	_WhipeChildViews(self)
	self.order = 1

	local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.cfg.questIndex)
	if (isHeader) then
		return
	end

	-- add item link or spacer
	if ((isComplete and isComplete == 1) or _GetAllObjectivesDone(self)) then
		if (self.cfg.ignoreable) then
			self:AddChildView(_GetIcon(self, "Interface\\GossipFrame\\ActiveQuestIcon", 0))
		else
			self:AddChildView(_GetIcon(self, "Interface\\GossipFrame\\ActiveQuestIcon", 3))
		end
	else
    	local link, icon, charges = GetQuestLogSpecialItemInfo(self.cfg.questIndex)
    	if (link) then
    		self:AddChildView(_GetQuestIcon(self, self.cfg.questIndex))
    	else
    		self:AddChildView(_GetSpacer(self))
    	end
    end
    
	-- add objectives
	self:AddChildView(self.objectives)
	
	-- add title
	
	local curQuestIndex = GetQuestLogSelection()
	SelectQuestLogEntry(self.cfg.questIndex) -- needed for next function
	local questDescription, questObjectives = GetQuestLogQuestText()
	SelectQuestLogEntry(curQuestIndex)
	
	self.cfg.questId = questID

	local fontCode
	if (self.profile.colorizeQuestDifficulty) then
		local colour = GetQuestDifficultyColor(level)
		fontCode = YakaGui:GetFontColorCode(colour)
	else
		fontCode = YakaGui:GetFontColorCode(unpack(self.profile.entryColor))
	end
	
	-- TODO separate title and status
	-- add quest title
--	if (isComplete) then
--		title = title.."|TInterface\\GossipFrame\\ActiveQuestIcon:0:aspectRatio|t "
--	end
	local extra


	-- add POI index
	local poiNo
	local poiButton = _QuestPOI_FindButtonByQuestId("WorldMapPOIFrame", questID)
	if (poiButton and poiButton.index) then
		poiNo = poiButton.index
	end
	
	local title
	if (poiNo) then
	 	title = "["..level.."/"..poiNo.."] "
	 else
	 	title = "["..level.."] "
	 end
	
	title = title..questTitle
		
	if (isComplete and isComplete == -1) then
		extra = " ("..FAILED..")"
	elseif (isComplete and isComplete == 1) then
		extra = " ("..COMPLETE..")"
	elseif (questTag) then
		extra = " ("..questTag..")"
	end
	
	if (not extra and isDaily) then
		extra = " ("..DAILY..")"
	end
	
	local tooltip = _CreateTooltip(title, questObjectives)
	
	local title = fontCode..title..FONT_COLOR_CODE_CLOSE
	if (self.cfg.ignoreable) then
		self.title:AddChildView(_GetCheckbox(self, questID))
		self.title:AddChildView(_GetText(self, title, tooltip, self.profile.entryPaddingLeft, self.profile.entryPaddingTop, true))
	else
		self.title:AddChildView(_GetText(self, title, tooltip, self.profile.entryPaddingLeft, self.profile.entryPaddingTop, true))
	end
	if (extra) then
		self.title:AddChildView(_GetText(self, extra, tooltip, 0, self.profile.entryPaddingTop, true, "fit"))
	end
	self.objectives:AddChildView(self.title)
	
	local n = GetNumQuestLeaderBoards(self.cfg.questIndex)
	for i = 1, n do
		local desc, objectiveType, done = GetQuestLogLeaderBoard(i, self.cfg.questIndex)
		if (desc and not done) then
			local code
			if (objectiveType ~= "spell") then
				if (self.profile.colorizeQuestObjectives) then
					code = _ColorizeObjective(self, desc)
				end
				desc = _WatchFrame_ReverseQuestObjective(desc)
			end
			if (not code) then
				code = YakaGui:GetFontColorCode(unpack(self.profile.objectiveColor))
			end
			local text = code..desc..FONT_COLOR_CODE_CLOSE
			self.objectives:AddChildView(_GetText(self, text, nil, self.profile.objectivePaddingLeft, self.profile.objectivePaddingTop))
		end
	end
end

local function _Init(self, cfgIn, parent)
	self:InitCfg(cfgIn)
	
	local cfg = self.cfg
	self.profile = qmstr.QuestMaster.db.profile
	cfg.questIndex = cfgIn.questIndex
	cfg.ignoreable = cfgIn.ignoreable
	self.order = 1
	
	cfg.layout = {
		layoutType = "LinearLayout",
		vertical = false
	}
	
	self.objectives = YakaGui:CreateView({
		viewType = "Container",
		order = 2,
		layout = {
			layoutType = "LinearLayout",
			vertical = true
		}
	})

	self.title = YakaGui:CreateView({
		viewType = "Container",
		order = 1,
		layout = {
			layoutType = "LinearLayout",
			vertical = false
		}
	})
	

--	self.info = YakaGui:CreateView({
--		viewType = "Container",
--		order = 2,
--		layout = {
--			layoutType = "LinearLayout",
--			vertical = false
--		}
--	})
	
	self:InitFrame(nil, parent)
	
	_Update(self)
end

--[[ 
	Creates a new instance.
--]]
function qmstr.QuestEntry:new(cfgIn, parent)
	local instance = setmetatable({}, self.metatable)
	_Init(instance, cfgIn, parent)
	return instance
end

function qmstr.QuestEntry.prototype:GetView()
	return self.view
end

function qmstr.QuestEntry.prototype:SetQuestIndex(questIndex, ignoreable)
	self.cfg.questIndex = questIndex
	self.cfg.ignoreable = ignoreable
	_Update(self)
end

-- register view

YakaGui:RegisterViewType(qmstr.QuestEntry)