--[[
	Copyright (C) Udorn (Blackhand)
--]]

local viewType = "QuestHeader"
local version = 1
local YakaGui = LibStub("YakaGui", true)
local info = YakaGui:GetViewInfo(viewType) 
if (info and info.version >= version) then 
	return 
end

qmstr = qmstr or {}
qmstr.QuestHeader = {}
qmstr.QuestHeader.prototype = {}
qmstr.QuestHeader.metatable = {__index = qmstr.QuestHeader.prototype}
setmetatable(qmstr.QuestHeader.prototype, {__index = yaka.Container.prototype})

qmstr.QuestHeader.info = {viewType = viewType, version = version}
qmstr.QuestHeader.GetViewInfo = function()
	return qmstr.QuestHeader.info
end

local ICON_SIZE = 16

local log = vendor.Debug:new("QuestHeader")

local function _SetCollapsed(info)
	local self = info.arg
	if (self.cfg.questIndex) then
    	if (info.value) then
    		log:Debug("collapse [%s]", self.cfg.questIndex)
    		CollapseQuestHeader(self.cfg.questIndex)
    	else
    		log:Debug("expand [%s]", self.cfg.questIndex)
    		ExpandQuestHeader(self.cfg.questIndex)
    	end
    end
end

local function _Init(self, cfgIn, parent)
	self:InitCfg(cfgIn)
	self.profile = qmstr.QuestMaster.db.profile
	local cfg = self.cfg
	
	cfg.questIndex = cfgIn.questIndex
	cfg.collapseable = cfgIn.collapseable
	self.order = 1
	
	cfg.layout = {
		layoutType = "LinearLayout",
		vertical = false
	}
	
	self.collapse = YakaGui:CreateView({
		viewType = "ToggleIcon",
		order = 1,
		width = ICON_SIZE,
		height = ICON_SIZE,
   		checkedNormalTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Plus-Up",
			texCoords = BUTTON_TEX_COORDS
		},
		checkedPushedTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Plus-Down",
			texCoords = BUTTON_TEX_COORDS
		},
		uncheckedNormalTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Minus-Up",
			texCoords = BUTTON_TEX_COORDS
		},
		uncheckedPushedTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Minus-Down",
			texCoords = BUTTON_TEX_COORDS
		},
		highlightTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Highlight",
			alphaMode = "ADD"
		},
		setValue = _SetCollapsed,
		arg = self

	})

	self.title = YakaGui:CreateView({
		viewType = "Text",
		order = 2,
		height = 12,
		width = "fill",
		font = "QuestMasterNormal"
	})

	self:InitFrame(nil, parent)
	
	self:SetQuestIndex(cfgIn.questIndex, cfgIn.collapseable)
end

--[[ 
	Creates a new instance.
--]]
function qmstr.QuestHeader:new(cfgIn, parent)
	local instance = setmetatable({}, self.metatable)
	_Init(instance, cfgIn, parent)
	return instance
end

function qmstr.QuestHeader.prototype:SetQuestIndex(questIndex, collapseable)
	
	self:RemoveChildViews()
	
	self.cfg.questIndex = questIndex
	if (questIndex and questIndex >= 1) then
    	local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.cfg.questIndex)
    	if (isHeader) then
    		self.cfg.title = questTitle
    		self.cfg.collapseable = collapseable
    		self.cfg.isCollapsed = isCollapsed
    		self.title:SetText(YakaGui:GetFontColorCode(unpack(self.profile.headerColor))..questTitle..FONT_COLOR_CODE_CLOSE)
    		
    		if (collapseable) then
    			self.collapse:SetChecked(isCollapsed)
   				self:AddChildView(self.collapse)
   			end
			self:AddChildView(self.title)
    	end
    end
end

-- register view

YakaGui:RegisterViewType(qmstr.QuestHeader)